/*
 * sound-plugin-hal-clovertrail
 *
 * Copyright (C) 2013  Intel Corporation. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

#include <alsa/asoundlib.h>
#include <errno.h>
#include <string.h>

#include <mm_debug.h>
#include <mm_error.h>

const char * card = "hw:4";

typedef struct {
    snd_ctl_elem_info_t *info;
    snd_ctl_elem_id_t *id;
    snd_ctl_elem_value_t *control;
}snd_stack_t;

static int prepare_snd_ctl_by_name(snd_ctl_t *handle, snd_stack_t *stack,
        snd_ctl_elem_type_t *type, unsigned int *count, const char* name)
{
	int err = 0;
	memset(stack->id, 0, snd_ctl_elem_id_sizeof());
	memset(stack->info, 0, snd_ctl_elem_info_sizeof());
	memset(stack->control, 0, snd_ctl_elem_value_sizeof());

	/* Set interface */
	snd_ctl_elem_id_set_interface(stack->id, SND_CTL_ELEM_IFACE_MIXER);

	if (NULL != name)
		snd_ctl_elem_id_set_name(stack->id, name);

	/* init info id */
	snd_ctl_elem_info_set_id(stack->info, stack->id);

	/* get info */
	err = snd_ctl_elem_info(handle, stack->info);
	if (err < 0) {
		debug_error("get info err : %s", snd_strerror(err));
		return -1;
	}

	/* get type */
	*type = snd_ctl_elem_info_get_type(stack->info);

	/* get count */
	*count = snd_ctl_elem_info_get_count(stack->info);

	/* set value id */
	snd_ctl_elem_value_set_id(stack->control, stack->id);
	return 0;
}

static int prepare_snd_ctl(snd_ctl_t *handle, snd_stack_t *stack,
            snd_ctl_elem_type_t *type, unsigned int *count, int numid)
{
    int err = 0;
    memset(stack->id, 0, snd_ctl_elem_id_sizeof());
    memset(stack->info, 0, snd_ctl_elem_info_sizeof());
    memset(stack->control, 0, snd_ctl_elem_value_sizeof());

    /* Set interface */
    snd_ctl_elem_id_set_interface(stack->id, SND_CTL_ELEM_IFACE_MIXER);

    if (numid > 0)
        snd_ctl_elem_id_set_numid(stack->id, numid);

    /* init info id */
    snd_ctl_elem_info_set_id(stack->info, stack->id);

    /* get info */
    err = snd_ctl_elem_info(handle, stack->info);
    if (err < 0) {
        debug_error("get info err : %s", snd_strerror(err));
        return -1;
    }

    /* get type */
    *type = snd_ctl_elem_info_get_type(stack->info);

    /* get count */
    *count = snd_ctl_elem_info_get_count(stack->info);

    /* set value id */
    snd_ctl_elem_value_set_id(stack->control, stack->id);
    return 0;
}

static int sound_card_hw_io(int set_flag, snd_ctl_t *handle, snd_stack_t *stack,
            snd_ctl_elem_type_t *type, unsigned int *count, int value, int *out_val)
{
    unsigned int idx;
    long tmp;
    int err = 0;

    if (!set_flag) {
        /* read element */
        err = snd_ctl_elem_read(handle, stack->control);
        if (err < 0) {
            debug_error("read element err :%s", snd_strerror(err));
            return -1;
        }

        for (idx = 0; idx < *count && idx < 128; idx++) {
            switch (*type)
            {
            case SND_CTL_ELEM_TYPE_BOOLEAN:
                *out_val = snd_ctl_elem_value_get_boolean(stack->control, idx);
                break;
            case SND_CTL_ELEM_TYPE_INTEGER:
                *out_val = snd_ctl_elem_value_get_integer(stack->control, idx);
                break;
            case SND_CTL_ELEM_TYPE_INTEGER64:
                *out_val = snd_ctl_elem_value_get_integer64(stack->control, idx);
                break;
            case SND_CTL_ELEM_TYPE_ENUMERATED:
                *out_val = snd_ctl_elem_value_get_enumerated(stack->control, idx);
                break;
            case SND_CTL_ELEM_TYPE_BYTES:
                *out_val = snd_ctl_elem_value_get_byte(stack->control, idx);
                break;
            default:
                debug_error("Unknown control element type");
                return -1;
            }
            debug_log("Read alsa element, idx=%d out_val=%d\n", idx, *out_val);
        }
    } else {
        /* set value */
        tmp = value;
        for (idx = 0; idx < *count && idx < 128; idx++)
        {
            switch (*type)
            {
            case SND_CTL_ELEM_TYPE_BOOLEAN:
                snd_ctl_elem_value_set_boolean(stack->control, idx, tmp);
                break;
            case SND_CTL_ELEM_TYPE_INTEGER:
                snd_ctl_elem_value_set_integer(stack->control, idx, tmp);
                break;
            case SND_CTL_ELEM_TYPE_INTEGER64:
                snd_ctl_elem_value_set_integer64(stack->control, idx, tmp);
                break;
            case SND_CTL_ELEM_TYPE_ENUMERATED:
                snd_ctl_elem_value_set_enumerated(stack->control, idx, tmp);
                break;
            case SND_CTL_ELEM_TYPE_BYTES:
                snd_ctl_elem_value_set_byte(stack->control, idx, tmp);
                break;
            default:
                debug_error("Unknown control element type");
                return -1;
            }

            debug_log("Write alsa element, idx=%u value=%ld\n", idx, tmp);
            /* Write element */
            err = snd_ctl_elem_write(handle, stack->control);
            if (err < 0) {
                debug_error("snd_ctl_elem_write err: %s", snd_strerror(err));
                /* Close sound control */
                return -1;
            }
        }
    }
    return 0;
}

static int sound_card_ctrl_get(int numid, int *out_val)
{
    snd_ctl_t *handle = NULL;
    snd_stack_t stack;
    snd_ctl_elem_type_t type;
    unsigned int count;
    int err = 0;

    snd_ctl_elem_id_alloca(&(stack.id));
    snd_ctl_elem_info_alloca(&(stack.info));
    snd_ctl_elem_value_alloca(&(stack.control));

    err = snd_ctl_open(&handle, card, 0);
    if (err < 0) {
        debug_error("control %s open err: %s", card, snd_strerror(err));
        return -1;
    }

    err = prepare_snd_ctl(handle, &stack, &type, &count, numid);
    if (err < 0) {
        debug_error("prepare hw err:%d", err);
        goto error;
    }

    err = sound_card_hw_io(0, handle, &stack, &type, &count, 0, out_val);
    if (err < 0) {
        debug_error("ctl hw io err:%d", err);
        goto error;
    }
    snd_ctl_close(handle);
    return 0;

error:
    snd_ctl_close(handle);
    return -1;
}

int sound_card_ctrl_set_by_name(const char *name, int value)
{
    snd_ctl_t *handle = NULL;
    snd_stack_t stack;
    snd_ctl_elem_type_t type;
    unsigned int count;
    int err = 0;

    snd_ctl_elem_id_alloca(&(stack.id));
    snd_ctl_elem_info_alloca(&(stack.info));
    snd_ctl_elem_value_alloca(&(stack.control));

    err = snd_ctl_open(&handle, card, 0);
    if (err < 0) {
        debug_error("control %s open err: %s", card, snd_strerror(err));
        return -1;
    }

    err = prepare_snd_ctl_by_name(handle, &stack, &type, &count, name);
    if (err < 0) {
        debug_error("prepare hw err:%d", err);
        goto error;
    }

    /* set value */
    err = sound_card_hw_io(1, handle, &stack, &type, &count, value, NULL);
    if (err < 0) {
        debug_error("ctl hw io err%d", err);
        goto error;
    }
    snd_ctl_close(handle);
    return 0;

error:
    snd_ctl_close(handle);
    return -1;
}

int sound_card_ctrl_set(int numid, int value)
{
    snd_ctl_t *handle = NULL;
    snd_stack_t stack;
    snd_ctl_elem_type_t type;
    unsigned int count;
    int err = 0;

    snd_ctl_elem_id_alloca(&(stack.id));
    snd_ctl_elem_info_alloca(&(stack.info));
    snd_ctl_elem_value_alloca(&(stack.control));

    err = snd_ctl_open(&handle, card, 0);
    if (err < 0) {
        debug_error("control %s open err: %s", card, snd_strerror(err));
        return -1;
    }

    err = prepare_snd_ctl(handle, &stack, &type, &count, numid);
    if (err < 0) {
        debug_error("prepare hw err:%d", err);
        goto error;
    }

    /* set value */
    err = sound_card_hw_io(1, handle, &stack, &type, &count, value, NULL);
    if (err < 0) {
        debug_error("ctl hw io err%d", err);
        goto error;
    }
    snd_ctl_close(handle);
    return 0;

error:
    snd_ctl_close(handle);
    return -1;
}

int set_dual_out(void )
{
    int err = 0;
    int headset_route;

    debug_fenter();

    /* set dual out: numid=35 "Speaker Mux Playback Route" 1,1 */
    /*//in ZTE phone, it's 38
    //err = sound_card_ctrl_set(35, 1);
    err = sound_card_ctrl_set_by_name("Speaker Mux Playback Route", 1);
    if (err < 0) {
        debug_error("CLV: failed to set alsa element for dual out");
        return 1;
    }
    debug_log("CLV: set dual out successfully");
*/
    debug_fleave();
    return 0;
}

